import logging
import os
from pathlib import Path
from typing import Final

import pytest

from tika_client.client import TikaClient

TIKA_URL: Final[str] = os.getenv("TIKA_URL", "http://localhost:9998")

SAMPLE_DIR: Final[Path] = Path(__file__).parent.resolve() / "samples"


@pytest.fixture()
def tika_client() -> TikaClient:
    with TikaClient(tika_url=TIKA_URL, log_level=logging.INFO) as client:
        yield client


@pytest.fixture()
def tika_client_compressed() -> TikaClient:
    with TikaClient(tika_url=TIKA_URL, log_level=logging.INFO, compress=True) as client:
        yield client



from datetime import datetime
from datetime import timedelta
from datetime import timezone

import magic
from pytest_httpx import HTTPXMock

from tests.conftest import SAMPLE_DIR
from tika_client.client import TikaClient
from tika_client.data_models import DublinCoreKey
from tika_client.data_models import TikaKey


class TestDateTimeFormat:
    def test_parse_offset_date_format_utc(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time with a UTC timezone in the +xx:yy format
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: "2023-05-17T16:30:44+00:00"},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created == datetime(
            year=2023,
            month=5,
            day=17,
            hour=16,
            minute=30,
            second=44,
            tzinfo=timezone.utc,
        )

    def test_parse_offset_date_format_zulu(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time with a UTC timezone in the Z format
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: "2023-01-17T16:35:44Z"},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created == datetime(
            year=2023,
            month=1,
            day=17,
            hour=16,
            minute=35,
            second=44,
            tzinfo=timezone.utc,
        )

    def test_parse_offset_date_format_positive(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time with a timezone in the +xx:yy format offset from UTC
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: "2023-06-17T16:30:44+08:00"},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created == datetime(
            year=2023,
            month=6,
            day=17,
            hour=16,
            minute=30,
            second=44,
            tzinfo=timezone(timedelta(hours=8)),
        )

    def test_parse_offset_date_format_negative(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time with a timezone in the -xx:yy format offset from UTC
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: "2023-06-17T16:30:44-08:00"},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created == datetime(
            year=2023,
            month=6,
            day=17,
            hour=16,
            minute=30,
            second=44,
            tzinfo=timezone(timedelta(hours=-8)),
        )

    def test_parse_offset_date_format_python_isoformat(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time with a timezone in the ISO 8061 format (as done by Python)
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        expected = datetime.now(tz=timezone.utc)

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: expected.isoformat()},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created == expected

    def test_parse_offset_date_no_match(self, tika_client: TikaClient, httpx_mock: HTTPXMock):
        """
        Test the datetime parsing properly handles a time string which doesn't match the correct formats
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"

        httpx_mock.add_response(
            json={TikaKey.ContentType: "test", TikaKey.Parsers: [], DublinCoreKey.Created: "202-06-17T16:30:44-0"},
        )

        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.created is None



from pathlib import Path
from typing import Final

from tika_client._types import MimeType
from tika_client._utils import BaseResource


class Metadata(BaseResource):
    """
    Handles interaction with the /meta endpoint of a Tika
    server REST API.

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-MetadataResource
    """

    ENDPOINT: Final[str] = "/meta"
    MULTI_PART_ENDPOINT = f"{ENDPOINT}/form"

    def from_file(self, filepath: Path, mime_type: MimeType = None):
        """
        PUTs the provided document to the metadata endpoint using multipart
        file encoding.  Optionally can provide the mime type
        """
        resp = self._put_multipart(self.MULTI_PART_ENDPOINT, filepath, mime_type)
        return self._decoded_response(resp)



import magic

from tests.conftest import SAMPLE_DIR
from tika_client.client import TikaClient


class TestRecursiveMetadataResource:
    def test_r_metadata_from_docx(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        documents = tika_client.rmeta.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in document.content
        assert document.created is None

    def test_r_metadata_from_docx_plain(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        documents = tika_client.rmeta.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "This is an DOCX test document, also made September 14, 2022" in document.content
        assert document.created is None

    def test_r_meta_microsoft_word_docx(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "microsoft-sample.docx"
        documents = tika_client.rmeta.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert (
            "<body><p>This is a sample document, generated by Microsoft Office on Wednesday, May 17, 2023.</p>\n<p>It is in English.</p>\n</body>"  # noqa: E501
            in document.content
        )

    def test_r_metadata_from_odt(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"
        documents = tika_client.rmeta.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 2
        document = documents[0]

        assert document.type == "application/vnd.oasis.opendocument.text"
        assert "<body><p>This is an ODT test document, created September 14, 2022</p>\n</body>" in document.content
        assert document.created is None

    def test_r_metadata_from_odt_plain(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"
        documents = tika_client.rmeta.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 2

        document = documents[0]
        assert document.type == "application/vnd.oasis.opendocument.text"
        assert "This is an ODT test document, created September 14, 2022" in document.content

        document = documents[1]
        assert document.type == "image/png"

    def test_r_metadata_from_ods_plain(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample-spreadsheet.ods"
        documents = tika_client.rmeta.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 2

        document = documents[0]
        assert "This is cell A1" in document.content
        assert "You sunk my battleship" in document.content

        document = documents[1]
        assert document.type == "image/png"

    def test_r_metadata_from_xlsx_plain(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample-spreadsheet.xlsx"
        documents = tika_client.rmeta.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert len(documents) == 1
        document = documents[0]

        assert "This is cell A1" in document.content
        assert "You sunk my battleship" in document.content



import logging
from types import TracebackType
from typing import Dict
from typing import Optional
from typing import Type

from httpx import Client

from tika_client._resource_meta import Metadata
from tika_client._resource_recursive import Recursive
from tika_client._resource_tika import Tika


class TikaClient:
    def __init__(
        self,
        *,
        tika_url: str,
        timeout: float = 30.0,
        log_level: int = logging.ERROR,
        compress: bool = False,
    ):
        # Configure the client
        self._client = Client(base_url=tika_url, timeout=timeout)

        # Set the log level
        logging.getLogger("httpx").setLevel(log_level)
        logging.getLogger("httpcore").setLevel(log_level)

        # Only JSON responses supported
        self._client.headers.update({"Accept": "application/json"})

        if compress:
            self._client.headers.update({"Accept-Encoding": "gzip"})

        # Add the resources
        self.metadata = Metadata(self._client, compress=compress)
        self.tika = Tika(self._client, compress=compress)
        self.rmeta = Recursive(self._client, compress=compress)

    def add_headers(self, header: Dict[str, str]) -> None:  # pragma: no cover
        """
        Updates the httpx Client headers with the given values
        """
        self._client.headers.update(header)

    def __enter__(self) -> "TikaClient":
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        self._client.close()



import logging
import re
from datetime import datetime
from datetime import timedelta
from datetime import timezone
from enum import Enum
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

# Based on https://cwiki.apache.org/confluence/display/TIKA/Metadata+Overview

logger = logging.getLogger("tika-client.data")
_TIME_RE = re.compile(
    r"(?P<year>\d{4})-"
    r"(?P<month>\d{2})-"
    r"(?P<day>\d{2})"
    r"[ tT]"
    r"(?P<hour>\d{2}):"
    r"(?P<minute>\d{2}):"
    r"(?P<second>\d{2})"
    r"(?P<fractional_seconds>\.\d+)?"
    r"(?P<timezone>[zZ]|[+-]\d{2}:\d{2})?",
)


class TikaKey(str, Enum):
    Parsers = "X-TIKA:Parsed-By"
    ContentType = "Content-Type"
    ContentLength = "Content-Length"
    Content = "X-TIKA:content"


class DublinCoreKey(str, Enum):
    Creator = "dc:creator"
    Created = "dcterms:created"
    Modified = "dcterms:modified"
    Rights = "dc:rights"
    Contributor = "dc:contributor"
    Title = "dc:title"
    Relation = "dc:relation"
    Type = "dc:type"
    Identifier = "dc:identifier"
    Publisher = "dc:publisher"
    Description = "dc:description"
    Subject = "dc:subject"
    Language = "dc:language"
    Format = "dc:format"


class XmpKey(str, Enum):
    About = "xmp:About"
    Created = "xmp:CreateDate"
    NumPages = "xmpTPg:NPages"


class OtherTikaKeys(str, Enum):
    CharacterCount = "meta:character-count"
    LastAuthor = "meta:last-author"
    Revision = "cp:revision"
    Language = "language"


class TikaResponse:
    """
    A basic response from the API.  It sets fields which the response
    always appears to have, and some small helpers for getting and converting
    other data types, including handling the chance those don't exist in the response.

    All returned data is available in the decoded JSON form under the .data attribute
    """

    def __init__(self, data: Dict) -> None:
        self.data = data
        # Always set keys
        self.type: str = self.data[TikaKey.ContentType]
        self.parsers: List[str] = self.data[TikaKey.Parsers]

        # Tika keys
        self.content = self.get_optional_string(TikaKey.Content)
        self.content_length = self.get_optional_int(TikaKey.ContentLength)

        # Dublin Core keys
        self.created = self.get_optional_datetime(DublinCoreKey.Created)
        self.modified = self.get_optional_datetime(DublinCoreKey.Modified)
        self.title = self.get_optional_string(DublinCoreKey.Title)

        # Xmp keys
        # TODO: Implement more of these
        self.xmp_created = self.get_optional_datetime(XmpKey.Created)
        self.page_count = self.get_optional_int(XmpKey.NumPages)

        # Other general keys
        self.character_count = self.get_optional_int(OtherTikaKeys.CharacterCount)
        self.revision = self.get_optional_int(OtherTikaKeys.Revision)
        self.language = self.get_optional_string(OtherTikaKeys.Language)
        self.last_author = self.get_optional_string(OtherTikaKeys.LastAuthor)

    # Helpers

    def get_optional_int(self, key: Union[TikaKey, DublinCoreKey, XmpKey, str]) -> Optional[int]:
        if key not in self.data:  # pragma: no cover
            return None
        return int(self.data[key])

    def get_optional_datetime(self, key: Union[TikaKey, DublinCoreKey, XmpKey, str]) -> Optional[datetime]:
        """
        If present, attempts to parse the given key as an ISO-8061 format
        datetime, including timezone handling and return if.

        If not present, return None
        """
        if key not in self.data:  # pragma: no cover
            return None

        date_str: str = self.data[key]

        m = _TIME_RE.match(date_str)
        if not m:
            return None

        (year, month, day, hour, minute, second, frac_sec, timezone_str) = m.groups()

        microseconds = int(float(frac_sec) * 1000000.0) if frac_sec is not None else 0
        tzinfo = None
        if timezone_str is not None:
            if timezone_str.lower() == "z":
                tzinfo = timezone.utc
            else:
                multi = -1 if timezone_str[0:1] == "-" else 1
                hours = int(timezone_str[1:3])
                minutes = int(timezone_str[4:])
                delta = timedelta(hours=hours, minutes=minutes) * multi
                tzinfo = timezone(delta)

        return datetime(
            year=int(year),
            month=int(month),
            day=int(day),
            hour=int(hour),
            minute=int(minute),
            second=int(second),
            microsecond=microseconds,
            tzinfo=tzinfo,
        )

    def get_optional_string(self, key: Union[TikaKey, DublinCoreKey, XmpKey, str]) -> Optional[str]:
        if key not in self.data:
            return None
        return self.data[key]

    def __repr__(self) -> str:  # pragma: no cover
        return f"{self.type} response"



from pathlib import Path
from typing import Final

from httpx import Client

from tika_client._types import MimeType
from tika_client._types import RequestContent
from tika_client._utils import BaseResource


class _TikaHtml(BaseResource):
    ENDPOINT: Final[str] = "/tika"
    MULTI_PART_ENDPOINT = "/tika/form"

    def from_file(self, filepath: Path, mime_type: MimeType = None):
        """
        Returns the formatted (as HTML) document data
        """
        return self._decoded_response(self._put_multipart(self.MULTI_PART_ENDPOINT, filepath, mime_type))

    def from_buffer(self, content: RequestContent, mime_type: MimeType = None):
        """
        Returns the HTML formatted document data from a given string of document content
        """
        return self._decoded_response(self._put_content(self.ENDPOINT, content, mime_type))


class _TikaPlain(BaseResource):
    PLAIN_TEXT_CONTENT: Final[str] = "/tika/text"
    MULTI_PART_PLAIN_TEXT_CONTENT = "/tika/form/text"

    def from_file(self, filepath: Path, mime_type: MimeType = None):
        """
        Returns the plain text document data
        """
        return self._decoded_response(self._put_multipart(self.MULTI_PART_PLAIN_TEXT_CONTENT, filepath, mime_type))

    def from_buffer(self, content: RequestContent, mime_type: MimeType = None):
        """
        Returns the plain text document data from a given string of document content
        """
        return self._decoded_response(self._put_content(self.PLAIN_TEXT_CONTENT, content, mime_type))


class Tika(BaseResource):
    """
    Handles interaction with the /tika endpoint of a Tika server REST API, returning the HTML
    formatted content or the plain text, depending on how the client is accessed

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-TikaResource
    """

    def __init__(self, client: Client, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        self.as_html = _TikaHtml(self.client, compress=compress)
        self.as_text = _TikaPlain(self.client, compress=compress)



import logging
import urllib.parse
from pathlib import Path
from typing import Dict

from httpx import Client

from tika_client._constants import MIN_COMPRESS_LEN
from tika_client._types import MimeType
from tika_client._types import RequestContent
from tika_client.data_models import TikaResponse

logger = logging.getLogger("tika-client.utils")


class BaseResource:
    def __init__(self, client: Client, *, compress: bool) -> None:
        self.client = client
        self.compress = compress

    def _put_multipart(self, endpoint: str, filepath: Path, mime_type: MimeType = None) -> Dict:
        """
        Given an endpoint, file and an optional mime type, does a multi-part form
        data upload of the file to the end point.

        Returns the JSON response of the server
        """
        with filepath.open("rb") as handle:
            if mime_type is not None:
                files = {"upload-file": (filepath.name, handle, mime_type)}
            else:
                files = {"upload-file": (filepath.name, handle)}  # type: ignore
            try:
                # Filename is valid ASCII, use it
                filepath.name.encode("ascii")
                content_header = {"Content-Disposition": f"attachment; filename={filepath.name}"}
            except UnicodeEncodeError:
                # Ignore non-ascii, in case RFC 5987 is not supported, but also encode it
                filename_safed = filepath.name.encode("ascii", "ignore").decode("ascii")
                filepath_quoted = urllib.parse.quote(filepath.name, encoding="utf-8")
                content_header = {
                    "Content-Disposition": f"attachment; filename={filename_safed}; filename*=UTF-8''{filepath_quoted}",
                }

            resp = self.client.post(endpoint, files=files, headers=content_header)
            resp.raise_for_status()
            # Always JSON
            return resp.json()

    def _put_content(self, endpoint: str, content: RequestContent, mime_type: MimeType = None) -> Dict:
        """
        Give, an endpoint, content and optional mime type, does an HTTP PUT with the given content.

        Returns the JSON response of the server
        """
        content_bytes = content.encode("utf-8") if isinstance(content, str) else content
        content_length = len(content_bytes)
        headers = {}
        if self.compress and content_length > MIN_COMPRESS_LEN:
            from gzip import compress

            content_bytes = compress(content_bytes)
            content_length = len(content_bytes)
            headers["Content-Encoding"] = "gzip"

        headers["Content-Length"] = str(content_length)
        if mime_type is not None:
            headers["Content-Type"] = mime_type

        resp = self.client.put(endpoint, content=content_bytes, headers=headers)
        resp.raise_for_status()
        return resp.json()

    @staticmethod
    def _decoded_response(resp_json: Dict):
        """
        If possible, returns a more detailed class with properties that appear often in this
        mime type.  Otherwise, it's a basically raw data response, but with some helpers
        for processing fields into Python types
        """
        return TikaResponse(resp_json)



import logging
from pathlib import Path
from typing import Final
from typing import List

from httpx import Client

from tika_client._types import MimeType
from tika_client._utils import BaseResource
from tika_client.data_models import TikaResponse

logger = logging.getLogger("tika-client.rmeta")


class _TikaRmetaBase(BaseResource):
    def _common_call(
        self,
        endpoint: str,
        filepath: Path,
        mime_type: MimeType = None,
    ) -> List[TikaResponse]:
        """
        Given a specific endpoint and a file, do a multipart put to the endpoint
        """
        documents: List[TikaResponse] = []
        for item in self._put_multipart(endpoint, filepath, mime_type):
            documents.append(self._decoded_response(item))
        return documents


class _RecursiveMetaHtml(_TikaRmetaBase):
    ENDPOINT: Final[str] = "/rmeta"
    MULTI_PART_ENDPOINT = "/rmeta/form/html"

    def from_file(self, filepath: Path, mime_type: MimeType = None):
        """
        Returns the formatted (as HTML) document data
        """
        return self._common_call(self.MULTI_PART_ENDPOINT, filepath, mime_type)


class _RecursiveMetaPlain(_TikaRmetaBase):
    ENDPOINT: Final[str] = "/rmeta/text"
    MULTI_PART_ENDPOINT = "/rmeta/form/text"

    def from_file(self, filepath: Path, mime_type: MimeType = None):
        """
        Returns the plain text document data
        """
        return self._common_call(self.MULTI_PART_ENDPOINT, filepath, mime_type)


class Recursive(BaseResource):
    """
    Handles interaction with the /rmeta endpoint of a Tika server REST API, returning the HTML
    formatted content or the plain text, depending on how the client is accessed

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-RecursiveMetadataandContent
    """

    def __init__(self, client: Client, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        # No support for XML endpoint.  Who wants that?
        self.as_html = _RecursiveMetaHtml(self.client, compress=compress)
        self.as_text = _RecursiveMetaPlain(self.client, compress=compress)



import magic

from tests.conftest import SAMPLE_DIR
from tika_client.client import TikaClient


class TestParseImageMetadata:
    def test_image_jpeg(self, tika_client: TikaClient):
        """
        Test the handling of a JPEG file metadata retrieval
        """
        test_file = SAMPLE_DIR / "sample.jpg"
        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "image/jpeg"

    def test_image_png(self, tika_client: TikaClient):
        """
        Test the handling of a PNG file metadata retrieval
        """
        test_file = SAMPLE_DIR / "sample.png"
        resp = tika_client.metadata.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "image/png"



import shutil
import tempfile
from pathlib import Path

import magic

from tests.conftest import SAMPLE_DIR
from tika_client.client import TikaClient


class TestParseFormatted:
    """
    Test the Tika endpoint for returning HTML formatted content
    """

    def test_parse_docx_from_file_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        resp = tika_client.tika.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content
        assert resp.content_length == 6424

    def test_parse_doc_from_file_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.doc"
        resp = tika_client.tika.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/msword"
        assert (
            "body><p>This is a test document, saved in the older .doc format for Word documents (but created in Google Drive)</p>\n</body>"  # noqa: E501
            in resp.content
        )
        assert resp.content_length == 23739
        assert resp.character_count == 90
        assert resp.page_count == 1
        assert resp.revision == 1
        assert resp.last_author == "cloudconvert_4"

    def test_parse_docx_no_mime_from_file_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        resp = tika_client.tika.as_html.from_file(test_file)

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content

    def test_parse_microsoft_word_docx_from_file_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "microsoft-sample.docx"
        resp = tika_client.tika.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert (
            "<body><p>This is a sample document, generated by Microsoft Office on Wednesday, May 17, 2023.</p>\n<p>It is in English.</p>\n</body>"  # noqa: E501
            in resp.content
        )

    def test_parse_odt_from_file_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"
        resp = tika_client.tika.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert "<body><p>This is an ODT test document, created September 14, 2022</p>\n</body>" in resp.content

    def test_parse_docx_from_buffer_as_html(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        content = test_file.read_bytes()
        resp = tika_client.tika.as_html.from_buffer(content, magic.from_buffer(content, mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content
        assert resp.content_length == 6183


class TestParsePlain:
    def test_parse_docx_from_file_as_text(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"

        resp = tika_client.tika.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "This is an DOCX test document, also made September 14, 2022" in resp.content

    def test_parse_odt_from_file_as_text(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"

        resp = tika_client.tika.as_text.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert "This is an ODT test document, created September 14, 2022" in resp.content


class TestParseContentPlain:
    def test_parse_docx_from_bytes_buffer(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"
        buffer = test_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert "This is an DOCX test document, also made September 14, 2022" in resp.content

    def test_parse_odt_from_bytes_buffer(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"
        buffer = test_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert "This is an ODT test document, created September 14, 2022" in resp.content

    def test_parse_odt_from_bytes_buffer_with_mime(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.odt"
        buffer = test_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer, "application/vnd.oasis.opendocument.text")

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert "This is an ODT test document, created September 14, 2022" in resp.content

    def test_html_document_from_string_buffer(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.html"
        buffer = test_file.read_text()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "text/html; charset=UTF-8"
        assert resp.parsers == ["org.apache.tika.parser.DefaultParser", "org.apache.tika.parser.html.HtmlParser"]
        assert "Hello world! This is HTML5 content in a file for" in resp.data["X-TIKA:content"]
        assert resp.data["dc:title"] == "This Is A Test"
        assert resp.data["description"] == "A sample HTML file"


class TestParseContentCompress:
    def test_parse_odt_from_bytes_buffer_compress(self, tika_client_compressed: TikaClient):
        test_file = SAMPLE_DIR / "test-document-images.odt"
        buffer = test_file.read_bytes()

        resp = tika_client_compressed.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.oasis.opendocument.text"


class TestFilenameContentDisposition:
    def test_non_ascii_filename(self, tika_client: TikaClient):
        test_file = SAMPLE_DIR / "sample.docx"

        with tempfile.TemporaryDirectory() as temp_dir:
            copy = shutil.copy(
                test_file,
                Path(temp_dir) / "Kostenerstattung für Meldebescheinigung Familienzuschlag.docx",
            )

            resp = tika_client.tika.as_text.from_file(copy, magic.from_file(str(copy), mime=True))

            assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
            assert "This is an DOCX test document, also made September 14, 2022" in resp.content



from tika_client.client import TikaClient
from tika_client.data_models import DublinCoreKey
from tika_client.data_models import TikaKey
from tika_client.data_models import XmpKey

__all__ = ["TikaClient", "TikaKey", "XmpKey", "DublinCoreKey"]



from datetime import datetime

import magic

from tests.conftest import SAMPLE_DIR
from tika_client.client import TikaClient


class TestLibreOfficeFormats:
    def test_parse_libre_office_writer_document(self, tika_client: TikaClient):
        """
        Test handling of a ODT document produced by LibreOffice
        """
        test_file = SAMPLE_DIR / "sample-libre-office.odt"
        resp = tika_client.tika.as_html.from_file(test_file, magic.from_file(str(test_file), mime=True))

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert (
            "<body><p>This is a document created by LibreOffice Writer 7.5.12, on July 19th, 2023</p>\n</body>"
            in resp.content
        )
        assert resp.content_length == 11149
        assert resp.created is not None
        assert resp.created == datetime(
            year=2023,
            month=7,
            day=19,
            hour=11,
            minute=30,
            second=44,
            microsecond=719000,
            tzinfo=None,
        )
